<?php

function set_freepbx_active_modules() {
    global $conf,$db,$active_modules;
    if(!$db->is_connected()) {
        $active_modules = array();
        return;
    }
    $database = $conf['DBNAME'];
    $query="select modulename from $database.modules where enabled=1";
    $res = $db->consulta($query);
    while($row=$db->fetch_assoc()) {
        if($row['modulename']=='core') { $row['modulename']="extensions"; }
        $active_modules[$row['modulename']]=1;   
    }
    $active_modules['trunks']=1;
}

function fop2_get_defplugin() {
    global $active_modules, $db;
    $results = $db->select("plugins","fop2templates","","isdefault=1");
    $pluginarray = explode(",",$results[0]['plugins']);
    return $pluginarray;
}

function fop2_get_defgroup() {
    global $active_modules, $db;
    $results = $db->select("groups","fop2templates","","isdefault=1");
    $grouparray = explode(",",$results[0]['groups']);
    return $grouparray;
}

function fop2_get_defperm() {
    global $active_modules, $db;
    $results = $db->select( "permissions","fop2templates","","isdefault=1");
    return $results[0]['permissions'];
}

function fop2_get_deftemplate() {
    global $active_modules, $db;
    $results = $db->select("id","fop2templates","","isdefault=1");
    return $results[0]['id'];
}

/*
// --------------------------------------------------------------------------------
// Iterates the freepbx array and get lables from there
// --------------------------------------------------------------------------------
*/
function system_all_values($field) {
    global $active_modules, $db;
    $return = array();

    $full_list = system_all_buttons();

    foreach ($full_list as $chan => $data) {
        if(isset($data[$field])) {
            $return[$chan]=trim($data[$field]);
        }
    }
    return $return;
}

/*
// --------------------------------------------------------------------------------
// Retrieves an array with buttons as returned by the backed system
// --------------------------------------------------------------------------------
*/
function system_all_buttons() {
    global $db, $config_engine, $conf;
    $return = array();

    if($config_engine=='freepbx') {
        $return = freepbx_check_extension_usage();
        // end freepbx system buttons
    } else if($config_engine == 'thirdlane_db') {
        $return = thirdlanedb_check_extension_usage();
    } else if($config_engine == 'thirdlane_old') {
        $return = thirdlaneold_check_extension_usage();
    } else if($config_engine == 'astdb') {

    }
    return $return;
}

/*
// --------------------------------------------------------------------------------
// Retrieves the full contents of the fop2buttons table to display on the edit page
// --------------------------------------------------------------------------------
*/
function fop2_all_buttons() {
    global $db;
    $sql = "SELECT * FROM fop2buttons ORDER BY sortorder";
    $fopbutton = array();
    $results = $db->consulta($sql);
    while($re = $db->fetch_assoc()) {
        $fopbutton[$re['device']] = array('type'=>$re['type'], 'exten'=>$re['exten'], 'name'=>$re['label'], 'devid'=>$re['id'],
                'context'=>$re['context'], 'mailbox'=>$re['mailbox'], 'privacy'=>$re['privacy'], 'group'=>$re['group'],
                'email'=>$re['email'], 'channel'=>$re['channel'], 'queuechannel'=>$re['queuechannel'], 'device'=>$re['device'], 
                'originatechannel'=>$re['originatechannel'], 'customastdb'=>$re['customastdb'], 'spyoptions'=>$re['spyoptions'], 
                'external' =>$re['external'], 'exclude' => $re['exclude'], 'tags' => $re['tags'], 'sortorder' => $re['sortorder']);
    }
    return $fopbutton;
}

function fop2_list_botones() {
    global $db;
    $ret = array();
    $results = $db->select("device","fop2buttons");
    if(is_array($results)) {
        foreach($results as $result) {
            $ret[$result['device']] = 1;
        }
    }
    return $ret;
}

function fop2_list_buttons() {
    global $db;
    $ret = array();
    $results = $db->select("id,type,exten,label","fop2buttons","","","type,exten");
    if(is_array($results)){
        foreach($results as $result) {
            $ret[] = $result;
        }
    }
    return $ret;
}

function fop2_custom_permissions() {
    global $db;
    $allowed = array();
    $results = $db->select("*","fop2permissions","","","name");
    if(is_array($results)){
        foreach($results as $result){
            // check to see if we have a dept match for the current AMP User.
            if (!isset($results['deptname']) || checkDept($result['deptname'])){
                $allowed[] = $result;
            }
        }
    }
    //if (isset($allowed)) {
    return $allowed;
    //} else {
    //    return null;
    //}
}

function fop2_list_templates() {
    global $db;
    $results = $db->select("*","fop2templates","","","name");
    if(is_array($results)){
        foreach($results as $result){
            $allowed[] = $result;
        }
    }
    if (isset($allowed)) {
        return $allowed;
    } else {
        return null;
    }
}

function fop2_list_templates_jsobject() {
    global $db;
    $results = $db->select("*","fop2templates","","","id");
    $finalret="";
    if(is_array($results)){
        foreach($results as $result) {
            $finalret .= "var tempperm_".$result['id']." = [];\n";
            $objperm = preg_split("/,/",$result['permissions']);
            foreach($objperm as $itm) {
                $finalret .= "tempperm_".$result['id'].".push('$itm')\n";
            }

            $objgrp = preg_split("/,/",$result['groups']);
            $finalret .= "var tempgrp_".$result['id']." = [];\n";
            foreach($objgrp as $itm) {
                $finalret .= "tempgrp_".$result['id'].".push('$itm')\n";
            }

            $objgrp = preg_split("/,/",$result['plugins']);
            $finalret .= "var tempplg_".$result['id']." = [];\n";
            foreach($objgrp as $itm) {
                $finalret .= "tempplg_".$result['id'].".push('$itm')\n";
            }
        }
    }
    return $finalret;
}



function fop2_list_users() {
    global $db;
    $results = $db->select("*","fop2users","","","exten");
    if(is_array($results)){
        foreach($results as $result){
            // check to see if we have a dept match for the current AMP User.
            if (!isset($results['deptname']) || checkDept($result['deptname'])){
                // return this item's dialplan destination, and the description
                $allowed[] = $result;
            }
        }
    }
    if (isset($allowed)) {
        return $allowed;
    } else {
        return null;
    }
}

function fop2_get_perm($id){
    global $db;
    //get all the variables for the meetme
    $results = $db->select("*","fop2permissions","","id='$id'");
    return $results[0];
}

function fop2_get_user($id){
    global $db;
    //get all the variables for the meetme
    $results = $db->select("*","fop2users","","id = '$id'");
    return $results[0];
}

function fop2_get_template($id){
    global $db;
    //get all the variables for the meetme
    $results = $db->select("*","fop2templates","","id = '$id'");
    return $results[0];
}

function fop2_chk_template($name) {
    global $db;
    $results = $db->select("id","fop2templates","","name = '$name'");
    if ( $results === false  ) {
        return false;
    } else {
        return true;
    }
}

function fop2_get_users() {
    global $db;
    $results = $db->select("exten","fop2users","","");
    if ( $results === false  ) {
        return false;
    } else {
        return $results;
    }
}

function fop2_chk_user($exten) {
    global $db;
    $results = $db->select("id","fop2users","","exten = '$exten'");
    if ( $results === false  ) {
        return false;
    } else {
        return true;
    }
}

function fop2_chk_perm($name) {
    global $db;
    $results = $db->select("id","fop2permissions","","name = '$name'");
    if ( $results === false  ) {
        return false;
    } else {
        return true;
    }
}

function fop2_chk_group($name) {
    global $db;
    $results = $db->select("id","fop2groups","","name = '$name'");
    if ( $results === false  ) {
        return false;
    } else {
        return true;
    }
}

function fop2_add_button($det) {
    global $db;
    $queuechannel = (isset($det['queuechannel']))?$det['queuechannel']:'';
    $customastdb  = (isset($det['customastdb']))?$det['customastdb']:'';
    $mailbox      = (isset($det['mailbox']))?$det['mailbox']:'';
    $email        = (isset($det['email']))?$det['email']:'';
    $group        = (isset($det['group']))?$det['group']:'';
    $external     = (isset($det['external']))?$det['external']:'';
    $context      = (isset($det['context']))?$det['context']:'';

    $results = $db->consulta(
        "REPLACE INTO fop2buttons (type, device, label, exten, context, mailbox, queuechannel, customastdb, email, `group`, external) ".
        "VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')",
        array($det['type'],$det['channel'],$det['name'],$det['exten'],$context,$mailbox,$queuechannel,$customastdb,$email,$group,$external)
    );
}

function fop2_add_perm($post) {

    global $db;
    extract($post);
    $name = trim(strtolower($name));

    if ( fop2_chk_perm($name) ) {
        echo "<script>javascript:alert(\"" . __("The permission name is already in use.") . "\"); history.back(-1);</script>";
        return false;
    }

    if ( $name=="" ) {
        echo "<script>javascript:alert(\"" . __("The name cannot be empty.") . "\"); history.back(-1);</script>";
        return false;
    }

    $allperm = strtolower($permissions);
    $allpermarr = preg_split("/\s+/",$allperm);
    $allpermarr = array_unique($allpermarr);
    $allpermarr = array_filter($allpermarr);

    if(in_array('all',$allpermarr)) {
        $strperm = "all";
    } else {
        $strperm = implode(",",$allpermarr);
    }

    $groups = fop2_list_groups(1);

    foreach ($groups as $d) {
        $groupid[$d['id']]=$d['name'];
    }

    $results = $db->consulta("INSERT INTO fop2permissions (name, permissions) VALUES ('$name', '$strperm')");

    if($results==1) {
        if(isset($post['includebot'])) {
            foreach($post['includebot'] as $idx=>$chan){
                $gname = $groupid[$chan];
                $results = $db->consulta("INSERT INTO fop2PermGroup (name,id_group,name_group) VALUES ('$name', '$chan','$gname')");
            }
        }
    }

    return true;
}


function fop2_add_user($post) {
    global $db;

    extract($post);

    if ( fop2_chk_user($userid) ) {
        echo "<script>alertify.error(\"" . __("The user number is already in use.") . "\"); history.back(-1);</script>";
        return false;
    }

    $allperm    = strtolower($permissions);
    $allperm    = preg_replace('/[^\s\da-z]/i', '', $allperm);
    $allpermarr = preg_split("/\s+/",$allperm);
    $allpermarr = array_unique($allpermarr);
    $allpermarr = array_filter($allpermarr);

    if(in_array('all',$allpermarr)) {
        $strperm = "all";
    } else {
        $strperm = implode(",",$allpermarr);
    }

    $results = $db->consulta("INSERT INTO fop2users (exten, secret, permissions) VALUES ('$userid', '$secret', '$strperm')");

    if($results==1) {
        if(isset($post['includebot'])) {
            foreach($post['includebot'] as $idx=>$chan){
                $results = $db->consulta("INSERT INTO fop2UserGroup (exten,id_group) VALUES ('$userid', '".$chan."')");
            }
        }
        if(isset($post['includeplugin'])) {
            foreach($post['includeplugin'] as $idx=>$chan){
                $results = $db->consulta("INSERT INTO fop2UserPlugin (exten,id_plugin) VALUES ('$userid', '".$chan."')");
            }
        }
 
    }
    return true;
}

function fop2_add_template($post) {
    global $db;

    extract($post);
 
    $templatename=trim($templatename);

    if ( fop2_chk_template($templatename) ) {
        echo "<script>alertify.error(\"" . __("The template name is already in use.") . "\"); history.back(-1);</script>";
        return false;
    }

    if ( $templatename=="" ) {
        echo "<script>alertify.error(\"" . __("The name cannot be empty.") . "\"); history.back(-1);</script>";
        return false;
    }

    $allperm = strtolower($permissions);
    $allpermarr = preg_split("/\s+/",$allperm);
    $allpermarr = array_unique($allpermarr);
    $allpermarr = array_filter($allpermarr);

    if(in_array('all',$allpermarr)) {
        $strperm = "all";
    } else {
        $strperm = implode(",",$allpermarr);
    }
    $mygroups = array();
    if(isset($post['includebot'])) {
        foreach($post['includebot'] as $idx=>$chan){
            $mygroups[] = $chan;
        }
    }
    $strgroup = implode(",",$mygroups);

    $myplugs = array();
    if(isset($post['includeplugin'])) {

        foreach($post['includeplugin'] as $idx=>$chan){
            $myplugs[] = $chan;
        }
    }
    $strplug = implode(",",$myplugs);
    $strplug = $db->escape_string($strplug);

    if(isset($makedefault)) {
        $mydef = 1;
        $results = $db->consulta("UPDATE fop2templates SET isdefault=0");
    } else {
        $mydef = 0;
    }
    $results = $db->consulta("INSERT INTO fop2templates (name, permissions, groups, plugins, isdefault) VALUES ('$templatename', '$strperm', '$strgroup','$strplug','$mydef')");
    return true;
}


function fop2_getDeviceId() {
    global $db;
    $myreturn = array();
    $results = $db->select("id,device","fop2buttons");
    if(is_array($results)){
        foreach($results as $result){
            $myreturn[$result['device']]=$result['id'];
        }
        return $myreturn;
    }
}

function fop2_get_perm_groups($name) {
    global $db;
    $ret = array();
    $results = $db->select("id_group","fop2PermGroup","","name='$name'");
    if(is_array($results)){
        foreach($results as $idx => $cols) {
            $ret[]=$cols['id_group'];
        }
        return $ret;
    } else {
        return array();
    }
}

function fop2_get_user_groups($exten) {
    global $db;
    $exten = preg_replace('/[^\d]/','',$exten);
    $ret = array();
    $results = $db->select("id_group","fop2UserGroup","","exten='$exten'");
    if(is_array($results)){
        foreach($results as $idx => $cols) {
            $ret[]=$cols['id_group'];
        }
        return $ret;
    } else {
        return array();
    }
}

function fop2_get_user_plugins($exten) {
    global $db;
    $exten = preg_replace('/[^\d]/','',$exten);
    $ret = array();
    $results = $db->select("id_plugin","fop2UserPlugin","","exten='$exten'");
    if(is_array($results)){
        foreach($results as $idx => $cols) {
            $ret[]=$cols['id_plugin'];
        }
        return $ret;
    } else {
        return array();
    }
}

function fop2_del_perm($id) {
    global $db;
    $id = preg_replace('/[^\d]/','',$id);
    $thisItem = fop2_get_perm($id);
    $name = $thisItem['name'];
    $results = $db->consulta("DELETE FROM fop2PermGroup WHERE name = '$name'");
    $results = $db->consulta("DELETE FROM fop2permissions WHERE id = '$id'");

    $results = $db->select("id,permissions","fop2users","","permissions like '%$name%'");
    if(is_array($results)){
        foreach($results as $idx => $perms) {
            $ret = array();
            $myid = $perms['id'];
            $mype = $perms['permissions'];
            $partperm = explode(",",$mype);
            foreach($partperm as $itemperm) {
                if($itemperm<>$name) {
                    $ret[] = $itemperm;
                }
            }
            $retstring = implode(",",$ret);
            $query = "UPDATE fop2users SET permissions='$retstring' WHERE id='$myid'";
            $results2 = $db->consulta($query);
        }
    }

    // Borramos custom permisos de template
    $results = $db->select("*","fop2templates","","permissions LIKE '%$name%'");
    if(is_array($results)){
        foreach($results as $result){
            $permitstring = preg_split("/,/",$result['permissions']);
            $finalperm = array();
            foreach($permitstring as $itemperm) {
                if($itemperm<>$name) {
                    $finalperm[]=$itemperm;
                }
            }
            $finalpermstring = implode(",",$finalperm);
            $db->consulta("UPDATE fop2templates SET permissions='$finalpermstring' WHERE id='".$result['id']."'");
        }
    }

}

function fop2_delete_template($id) {
    global $db;
    $id = preg_replace('/[^\d]/','',$id);
    $results = $db->select("name","fop2templates","","id='$id'");
    $db->consulta("DELETE FROM fop2templates WHERE id = '$id'","query");
    return $results[0];
}

function fop2_del_user($id) {
    global $db;
    $id = preg_replace('/[^\d]/','',$id);
    $thisItem = fop2_get_user($id);
    $exten = $thisItem['exten'];
    $results = $db->consulta("DELETE FROM fop2UserGroup WHERE exten = '$exten'");
    $results = $db->consulta("DELETE FROM fop2UserPlugin WHERE exten = '$exten'");
    $results = $db->consulta("DELETE FROM fop2users WHERE id = '$id'");
}

function fop2_buttonsrefresh($post) {
    global $db;
    $full_list = system_all_values('name');
    foreach($full_list as $chan => $label) {
        $label = $db->escape_string($label);
        $query="UPDATE fop2buttons SET label='$label' WHERE device='$chan'";
        $results = $db->consulta($query); 
    }
}

function fop2_fieldedit($post) {
    global $db;
    $partes = split("_",$post['field']);
    $field = $partes[0];
    $id    = $partes[1];
    $val   = $db->escape_string($post['value']);
    if($field<>"name" && $field<>"privacy" && $field<>"channel" && $field<>"group" && $field<>"email" && $field<>'queuechannel' && $field<>'originatechannel' && $field<>'customastdb' && $field<>'spyoptions' && $field<>'external' && $field<>'excludebot' && $field<>'tags' && $field<>'mailbox') { die($field); }
    if($field=="name") $field="label";
    if($field=="excludebot") $field="exclude";
    $results = $db->consulta("UPDATE fop2buttons SET `$field`='$val' WHERE id='$id'");
    return $results;
}

function fop2_buttonsedit($post) {
    global $db;

    $exclude=array();
    foreach($post as $key=>$val) {
        if(ereg("_",$key)) {
            $partes = split("_",$key);
            $field = $partes[0];
            $id    = $partes[1];
            $exclude[$id]=0;
        }
    }
    foreach($post as $key=>$val) {
        if(ereg("_",$key)) {
            $partes = split("_",$key);
            $field = $partes[0];
            $id    = $partes[1];
            $val   = $db->escape_string($val);
            if($field<>"name" && $field<>"privacy" && $field<>"channel" && $field<>"group" && $field<>"email" && $field<>'queuechannel' && $field<>'originatechannel' && $field<>'customastdb' && $field<>'spyoptions' && $field<>'external' && $field<>'excludebot' && $field<>'tags') { die($field); }
            if($field=="name") $field="label";
            if($field=="excludebot") {
                $exclude[$id]=1;
            } else {
                $results = $db->consulta("UPDATE fop2buttons SET `$field`='$val' WHERE id='$id'");
            }
        }
    }
    foreach($exclude as $id=>$val) {
        $results = $db->consulta("UPDATE fop2buttons SET exclude='$val' WHERE id='$id'");
    }
}

function fop2_edit_perm($id, $post) {
    global $db;
    $id = preg_replace('/[^\d]/','',$id);
    $thisItem = fop2_get_perm($id);
    $name = $thisItem['name'];

    extract($post);

    $allperm = strtolower($permissions);
    $allpermarr = preg_split("/\s+/",$allperm);
    $allpermarr = array_unique($allpermarr);
    $allpermarr = array_filter($allpermarr);

    if(in_array('all',$allpermarr)) {
        $strperm = "all";
    } else {
        $strperm = implode(",",$allpermarr);
    }

    $results = $db->consulta("UPDATE fop2permissions SET permissions='$strperm' WHERE id = '$id'");

    $results = $db->consulta("DELETE FROM fop2PermGroup WHERE name = '$name'");

    $groups = fop2_list_groups(1);
    foreach ($groups as $d) {
        $groupid[$d['id']]=$d['name'];
    }

    if(isset($post['includebot'])) {
        foreach($post['includebot'] as $idx=>$chan){
            $gname = $groupid[$chan];
            $results = $db->consulta("INSERT INTO fop2PermGroup (name,id_group,name_group) VALUES ('$name', '$chan','$gname')");
        }
    }
    return true;
}

function fop2_edit_template($id, $post) {
    global $db;
    $id = preg_replace('/[^\d]/','',$id);
    $thisItem = fop2_get_template($id);
    $name = $thisItem['name'];

    extract($post);

    $allperm = strtolower($permissions);
    $allpermarr = preg_split("/\s+/",$allperm);
    $allpermarr = array_unique($allpermarr);
    $allpermarr = array_filter($allpermarr);

    if(in_array('all',$allpermarr)) {
        $strperm = "all";
    } else {
        $strperm = implode(",",$allpermarr);
    }
    $mygroups = array();
    if(isset($post['includebot'])) {
        foreach($post['includebot'] as $idx=>$chan){
            $mygroups[] = $chan;
        }
    }
    $strgroup = implode(",",$mygroups);

    $myplugs = array();
    if(isset($post['includeplugin'])) {
        foreach($post['includeplugin'] as $idx=>$chan){
            $myplugs[] = $chan;
        }
    }
    $strplug = implode(",",$myplugs);
    $strplug = $db->escape_string($strplug);

    if(isset($makedefault)) {
        $mydef = 1;
        $results = $db->consulta("UPDATE fop2templates SET isdefault=0");
    } else {
        $mydef = 0;
    }

    $results = $db->consulta("UPDATE fop2templates SET groups = '$strgroup', permissions='$strperm', plugins='$strplug', isdefault='$mydef' WHERE id = '$id'");

    return true;
}

function fop2_edit_user($id, $post) {
    global $db;
    $id = preg_replace('/[^\d]/','',$id);
    $thisItem = fop2_get_user($id);
    $exten = $thisItem['exten'];

    extract($post);

    $allperm    = strtolower($permissions);
    $allpermarr = preg_split("/\s+/",$allperm);
    $allpermarr = array_unique($allpermarr);
    $allpermarr = array_filter($allpermarr);

    if(in_array('all',$allpermarr)) {
        $strperm = "all";
    } else {
        $strperm = implode(",",$allpermarr);
    }

    $results = $db->consulta("UPDATE fop2users SET secret = '$secret', permissions='$strperm' WHERE id = '$id'");
    $results = $db->consulta("DELETE FROM fop2UserGroup WHERE exten = '$exten'");
    $results = $db->consulta("DELETE FROM fop2UserPlugin WHERE exten = '$exten'");

    if(isset($post['includebot'])) {
        foreach($post['includebot'] as $idx=>$chan){
            $results = $db->consulta("INSERT INTO fop2UserGroup (exten,id_group) VALUES ('$exten', '$chan')");
        }
    }
    if(isset($post['includeplugin'])) {
        foreach($post['includeplugin'] as $idx=>$chan){
            $results = $db->consulta("INSERT INTO fop2UserPlugin (exten,id_plugin) VALUES ('$exten', '$chan')");
        }
    }
 
    return true;
}

function fop2_get_next_available_exten() {
    global $db;
    $valid_exten = array();
    $results = system_all_values('exten');
    $types   = system_all_values('type');
    foreach($results as $key=>$val) {
        if($types[$key]=='extension') {
            $valid_exten[]=$val;
        }
    }
    asort($valid_exten);

    $existent_exten = array();
    $results = $db->select("exten","fop2users");
    if(is_array($results)){
        foreach($results as $result){
            $existent_exten[]=$result['exten'];
        }
    }
    $disponibles = array_diff($valid_exten,$existent_exten);
    asort($disponibles);

    $proximodisponible = array_shift($disponibles);

    return $proximodisponible;
}

function thirdlaneold_check_extension_usage() {
    global $db, $astman, $conf;
    $extenlist = array();

    // Extensions

    if(!is_readable("/etc/asterisk/users.txt")) { die('Cannot read /etc/asterisk/users.txt file'); }

    $voicemail_pins = read_voicemail_conf_pins("/etc/asterisk/voicemail.conf");

    $lines = file("/etc/asterisk/users.txt");
    $exten='';
    $extenlist = array();
    foreach($lines as $line) {
        $line=trim($line);
        //echo $line."<br>";
        if(preg_match("/^\[/",$line)) {
            if(isset($data['channel'])) {
                $mbox = $data['mailbox'];
                $lastname = isset($data['last_name'])?$data['last_name']:'';
                $firstname = isset($data['first_name'])?$data['first_name']:'';
                $data['name']=$lastname." ".$firstname;
                $data['type']='extension';
                $data['context']='from-inside';
                $data['vmpin'] = isset($voicemail_pins[$mbox])?$voicemail_pins[$mbox]:$data['exten'];
                $extenlist[$data['channel']] = $data;
            }
            $exten = substr($line,1,-1);
            $data = array();
        } else {
            if($exten<>'') { 
                $partes = preg_split("/=/",$line);
                $param = $partes[0];
                $value = isset($partes[1])?trim($partes[1]):'';
                if($param=='first_name') {
                    $data['first_name'] = $value;
                } else
                if($param=='last_name') {
                    $data['last_name'] = $value;
                } else
                if($param=='email') {
                    $data['email'] = $value;
                } else
                if($param=='phones') {
                    $data['channel'] = $value;
                } else
                if($param=='mailbox') {
                    $data['mailbox'] = $value."@default";
                } else
                if($param=='department') {
                    $data['group'] = $value;
                } else
                if($param=='ext') {
                    $data['exten'] = $value;
                }
            }
        }
    }
    // last item
    $data['type']='extension';
    $data['context']='from-inside';
    $lastname = isset($data['last_name'])?$data['last_name']:'';
    $firstname = isset($data['first_name'])?$data['first_name']:'';
    $data['name']=$lastname." ".$firstname;
    $data['vmpin'] = isset($voicemail_pins[$mbox])?$voicemail_pins[$mbox]:$data['exten'];
    $extenlist[$data['channel']] = $data;

    // Queues
    if(is_readable("/etc/asterisk/queues.conf")) { 
        $lines = file("/etc/asterisk/queues.conf");
        $data = array();
        $cont=0;
        foreach($lines as $line) {
            $line=trim($line);
            if(preg_match("/^\[/",$line)) {
                $cont++;
                $queue = substr($line,1,-1);
                $data['channel'] = "QUEUE/$queue";
                $data['type']    = "queue";
                $data['context'] = "default";
                $data['name']    = $queue;
                $data['exten']   = $cont;
                $extenlist[$data['channel']] = $data;
            }
        }
    }

    // Conferences 
    if(is_readable("/etc/asterisk/meetme.txt")) {
        $lines = file("/etc/asterisk/meetme.txt");
        $data = array();
        $cont=0;
        foreach($lines as $line) {
            $line=trim($line);

            if(preg_match("/^\[/",$line)) {
                if(isset($data['channel'])) {
                    $data['type']    = 'conference';
                    $data['context'] = 'from-inside';
                    $extenlist[$data['channel']] = $data;
                }
                $exten = substr($line,1,-1);
                $data = array();
            } else {
                if($exten<>'') { 
                    $partes = preg_split("/=/",$line);
                    $param = $partes[0];
                    $value = isset($partes[1])?$partes[1]:'';
                    if($param=='description') {
                        $data['name'] = $value;
                    } else
                    if($param=='conference') {
                        $data['exten'] = $value;
                        $data['channel'] = "CONFERENCE/".$value;
                    }
                }
            }
        }
        // last item
        $data['type']    = 'conference';
        $data['context'] = 'from-inside';
        $extenlist[$data['channel']] = $data;
    }

    // Trunks
    if(is_readable("/etc/asterisk/trunks.include")) {
        $cont = 0;
        $lines = file("/etc/asterisk/trunks.include");
        foreach($lines as $line) {
            $line=trim($line);
            $partes = preg_split("/=/",$line);
            $param = $partes[0];
            $value = isset($partes[1])?$partes[1]:'';
            if(preg_match("/^TRUNK_NAME/",$line)) {
                $data['name']=$value;
            } else
            if(preg_match("/^TRUNK_PROTOCOL/",$line)) {
                $data['channel']=$value."/".$data['name'];
            } else
            if(preg_match("/^TRUNK_DIAL/",$line)) {
                $cont++;
                $data['type']='trunk';
                $data['exten']=$cont;
                $extenlist[$data['channel']] = $data;
                $data = array();
            }
        }
    }

    $conf["MGRPORT"]='5038';
    $conf["MGRUSER"]='internal';
    $conf["MGRPASS"]='insecure';

    if(!$res = $astman->connect("127.0.0.1:".$conf["MGRPORT"], $conf["MGRUSER"] , $conf["MGRPASS"], 'off')) {
        unset($astman);
    }

    if($astman) {
        $get = $astman->Command('dialplan show parkedcalls');
        $get = array_shift($get);
        $lines = preg_split("/\n/",$get);
        foreach($lines as $line) {
            $line = trim($line);
            if(preg_match("/Context/",$line)) {
                $partes = preg_split("/ /",$line);
                $context = substr(trim($partes[2]),1,-1);
            }
            if(preg_match("/=>/",$line)) {
                $partes = preg_split("/=>/",$line);
                $park_exten = substr(trim($partes[0]),1,-1);
                if($context=='parkedcalls') {
                    $channel = "PARK/default";
                    $name = 'Default';
                } else {
                    $partes = preg_split("/_/",$context);
                    $channel = "PARK/parkinglot_".$partes[1];
                    $name = $partes[1];
                }
                $data = array();
                $data['name']    = $name;
                $data['channel'] = $channel;
                $data['type']    = 'park';
                $data['exten']   = $park_exten;
                $data['context'] = $context;
                $extenlist[$data['channel']]  = $data;
            }
        }
    }

    return $extenlist;

}

function thirdlanedb_check_extension_usage() {

    global $db, $astman, $conf;
    $extenlist = array();


    // Extensions

    $fields = "phones AS dial, u.email AS email, CONCAT(last_name,' ',first_name) AS name, ";
    $fields.= "u.mobile AS external, 'from-inside' AS context, u.ext as extension, ";
    $fields.= "CONCAT(v.mailbox,'@',v.context) AS mailbox, d.department AS `group`";

    $joins = "LEFT JOIN voicemail_users v ON u.mailbox=v.mailbox ";
    $joins.= "LEFT JOIN directory d ON u.ext=d.ext";

    $results = $db->select($fields,"user_extensions u",$joins,"","name");

    if(is_array($results)) {
        foreach ($results as $result) {
            $data = array();
            $data['type']    = 'extension';
            $data['name']    = $result['name'];
            $data['channel'] = $result['dial'];
            $data['mailbox'] = $result['mailbox'];
            $data['context'] = $result['context'];
            $data['email']   = $result['email'];
            $data['group']   = $result['group'];
            $data['exten']   = $result['extension'];
            $data['external']= $result['external'];
            $extenlist[$data['channel']]  = $data;
        }
    }

    // Queues
    $fields = "CONCAT('QUEUE/',name) AS dial,'default' AS context,id AS extension,CONCAT('Queue ',name) AS name";
    $results = $db->select($fields,'queues');
    if(is_array($results)) {
        foreach ($results as $result) {
            $data = array();
            $data['type']    = 'queue';
            $data['name']    = $result['name'];
            $data['context'] = $result['context'];
            $data['channel'] = $result['dial'];
            $data['exten']   = $result['extension'];
            $extenlist[$data['channel']]  = $data;
        }
    }
 
    // Conferences
    $fields = "CONCAT('CONFERENCE/',name) AS dial,'default' AS context,id AS extension,CONCAT('Conference ',name) AS name ";
    $results = $db->select($fields,'conferences');
    if(is_array($results)) {
        foreach ($results as $result) {
            $data = array();
            $data['type']    = 'conference';
            $data['name']    = $result['name'];
            $data['context'] = $result['context'];
            $data['channel'] = $result['dial'];
            $data['exten']   = $result['extension'];
            $extenlist[$data['channel']]  = $data;
        }
    }

    // Trunks
    $fields = "CONCAT(protocol,name) AS dial,'default' AS context,id AS extension,name ";
    $results = $db->select($fields,'trunks');
    if(is_array($results)) {
        foreach ($results as $result) {
            $data = array();
            $data['type']    = 'trunk';
            $data['name']    = $result['name'];
            $data['context'] = $result['context'];
            $data['channel'] = $result['dial'];
            $data['exten']   = $result['extension'];
            $extenlist[$data['channel']]  = $data;
        }
    }

    if(!$res = $astman->connect("127.0.0.1:".$conf["MGRPORT"], $conf["MGRUSER"] , $conf["MGRPASS"], 'off')) {
        unset($astman);
    }

    if($astman) {
        $get = $astman->Command('dialplan show parkedcalls');
        $get = array_shift($get);
        $lines = preg_split("/\n/",$get);
        foreach($lines as $line) {
            $line = trim($line);
            if(preg_match("/Context/",$line)) {
                $partes = preg_split("/ /",$line);
                $context = substr(trim($partes[2]),1,-1);
            }
            if(preg_match("/=>/",$line)) {
                $partes = preg_split("/=>/",$line);
                $park_exten = substr(trim($partes[0]),1,-1);
                if($context=='parkedcalls') {
                    $channel = "PARK/default";
                    $name = 'Default';
                } else {
                    $partes = preg_split("/_/",$context);
                    $channel = "PARK/parkinglot_".$partes[1];
                    $name = $partes[1];
                }
                $data = array();
                $data['name']    = $name;
                $data['channel'] = $channel;
                $data['type']    = 'park';
                $data['exten']   = $park_exten;
                $data['context'] = $context;
                $extenlist[$data['channel']]  = $data;
            }
        }
    }

    return $extenlist;

}

/*
Checks for all available modules and run a check_extension_freepbx function
to retrieve a hash with all extensions and data for a complete set of buttons
so we can compare with the fop2buttons table and remove what was deleted and
insert what is new.
*/
function freepbx_check_extension_usage($exten=true, $module_hash=false) {

    global $active_modules;

    $exten_usage   = array();
    $exten_matches = array();

    if (!is_array($module_hash)) {
        $module_hash = $active_modules;
    }

    if (!is_array($exten) && $exten !== true) {
        $exten = array($exten);
    }

    foreach(array_keys($module_hash) as $mod) {
        $function = $mod."_check_extensions_freepbx";
        if (function_exists($function)) {
            $module_usage = $function($exten);
            if (!empty($module_usage)) {
                $exten_usage = array_merge($module_usage, $exten_usage);
            }
        } 
    }
    if ($exten === true) {
        return $exten_usage;
    } else {
        foreach ($exten_usage as $chan=>$data) {
            foreach ($exten as $idx=>$test_exten) {
                if ($data['exten']==$test_exten) {
                    $exten_matches[$chan] = $data;
                }
            }
        }
    }
    return $exten_matches;
}

function trunks_check_extensions_freepbx() {
    global $conf,$db;
    $database = $conf['DBNAME'];
        
    $sql = "SELECT `trunkid` , `tech` , `channelid` , `disabled` FROM $database.`trunks` ORDER BY `trunkid`";
    $trunks = $db->consulta($sql);
    $extenlist = array();

    while($trunk = $db->fetch_assoc()) {
        $trunk_id = "OUT_".$trunk['trunkid'];
        $disabled = $trunk['disabled'];
        $tech = strtoupper($trunk['tech']);
        switch ($tech) {
            case 'IAX':
                $dialstring = 'IAX2/'.$trunk['channelid'];
                break;
            case 'CUSTOM':
                $dialstring = $trunk['channelid'];
                break;
            default:
                $dialstring = $tech.'/'.$trunk['channelid'];
                break;
        }
        $thisexten = "OUT_".$trunk['trunkid'];
        $dialstring = preg_replace('/(.*)\/\$OUTNUM\$(.*)?/','$1',trim($dialstring));
        /*
        $extenlist[$thisexten]['name']   = $dialstring;
        $extenlist[$thisexten]['channel'] = $dialstring;
        $extenlist[$thisexten]['type']    = "trunk";
        $extenlist[$thisexten]['exten']   = $thisexten;
        */
        $data = array();
        $data['name']    = $dialstring;
        $data['channel'] = $dialstring;
        $data['type']    = "trunk";
        $data['exten']   = $thisexten;
        $data['context'] = '';
        $extenlist[$dialstring]=$data;

    }
    return $extenlist;
}

function ringgroups_check_extensions_freepbx($exten=true) {
    global $conf, $db;
    $database = $conf['DBNAME'];
    $extenlist=array();
    $sql = "SELECT `grpnum`, `description` FROM ringgroups ORDER BY CAST(grpnum as UNSIGNED)";
    $res = $db->consulta($sql);
    while($result = $db->fetch_assoc()) { 
        $thisexten = $result['grpnum'];
        $data = array();
        $data['name'] = $result['description'];
        $data['channel'] = "RINGGROUP/$thisexten";
        $data['type']    = "ringgroup";
        $data['exten']   = $thisexten;
        $extenlist[$data['channel']]  = $data;
    }
    return $extenlist;
}

function extensions_check_extensions_freepbx($exten=true) {

    global $conf, $db, $fpbx_version;
    $deviceFor = array();
    $database = $conf['DBNAME'];

    $queue_conf = freepbx_get_queues();

    $voicemail_pins = read_voicemail_conf_pins("/etc/asterisk/voicemail.conf");

    if($fpbx_version <= 260) {
        $qcontext="from-internal";
    } else {
        $qcontext="from-queue";
    }

    if ($conf['EXTENSIONS'] == "deviceanduser") {
        $fields="extension,concat('USER/',extension) AS dial, if(voicemail='novm','',concat(extension,'@',voicemail)) AS mailbox,name";
    } else {
        $fields  = "extension, name, IF(dial IS null,CONCAT('VIRTUAL/',extension),dial) AS dial, ";
        $fields .= "IF(voicemail='novm','',CONCAT(extension,'@',voicemail)) AS mailbox ";
    }
    $join = "LEFT JOIN $database.devices ON users.extension=devices.id";

    $extenlist = array();
    if (is_array($exten) && empty($exten)) {
        return $extenlist;
    }

    $where='';
    if (is_array($exten)) {
        $where = "extension in ('".implode("','",$exten)."')";
    }

    $results = $db->select($fields,"$database.users",$join,$where,"","","");
    if(is_array($results)) {
        foreach ($results as $result) {

            if(preg_match("/[,&]/",$result['dial'])) {
                $partes = preg_split("/[,&]/",$result['dial']);
                $result['dial']=$partes[0];
            }

            $thisexten = $result['extension'];
            $qchannel  = freepbx_construct_queuechannel($thisexten,$result['dial'],$result['name'],$qcontext,$queue_conf);
            $vmpin     = isset($voicemail_pins[$result['mailbox']])?$voicemail_pins[$result['mailbox']]:$thisexten;
            $data = array();
            $data['name'] = $result['name'];
            $data['channel'] = $result['dial'];
            $data['mailbox'] = $result['mailbox'];
            $data['type']    = 'extension';
            $data['context'] = 'from-internal';
            $data['exten']   = $thisexten;
            $data['vmpin']   = $vmpin;
            $data['customastdb'] = 'CF/'.$thisexten;
            $data['queuechannel'] = $qchannel;
            $extenlist[$data['channel']]  = $data;

            $deviceFor[$thisexten]=$result['dial'];

        }
    }

    // Ahora que tenemos devices, nos fijamos restrictqueue

    $restrictstring="";
    foreach($queue_conf as $queue=>$data) {
        if($data['dynmemberonly']=='yes') {
            if(isset($data['dynmembers'])) {
                $datarray=array();
                foreach($data['dynmembers'] as $member) {
                    $pen_pos = strrpos($member, ",");
                    if ( $pen_pos !== false ) {
                        $thisexten = substr($member,0,$pen_pos); 
                    } else {
                        $thisexten = $member;
                    }
                    $thisexten = preg_replace("/[^0-9#*]/", "", $thisexten); 
                    $datarray[]=$deviceFor[$thisexten];
                }
                $restrictstring .= "restrictqueue=QUEUE/$queue:".implode(",",$datarray)."\\n";
            }
        } 
    }
    if($restrictstring <> '') {
        $query = "REPLACE INTO fop2settings (keyword,value) VALUES ('restrictqueue','$restrictstring')";
        $db->consulta($query);
    }

    return $extenlist;
}

function conferences_check_extensions_freepbx($exten=true) {
    global $conf,$db;
    $database = $conf['DBNAME'];
    $extenlist = array();
    if (is_array($exten) && empty($exten)) {
        return $extenlist;
    }
    $where='';
    if (is_array($exten)) {
        $where.= "exten in ('".implode("','",$exten)."')";
    }
    $results = $db->select("exten,description","$database.meetme","",$where,"","","exten");

    if(is_array($results)) {
        foreach ($results as $result) {
            $thisexten = $result['exten'];
            $data = array();
            $data['name'] = $result['description'];
            $data['channel'] = "CONFERENCE/$thisexten";
            $data['context'] = "ext-meetme";
            $data['type']    = 'conference';
            $data['exten']   = $thisexten;
            $extenlist[$data['channel']]  = $data;
        }
    }
    return $extenlist;
}

function parking_check_extensions_freepbx($exten=true) {

    global $db, $conf;
    $database = $conf['DBNAME'];
    $extenlist = array();

    $resu  = $db->consulta("DESC $database.parkinglot");
    if($db->error()=='') {
        $results = $db->select("data","$database.parkinglot","","keyword='parkext'");
        if(is_array($results)) {
            foreach ($results as $result) {
                $thisexten = $result['data'];
                $data = array();
                $data['name']   = "Default";
                $data['channel'] = "PARK/default";
                $data['type']    = 'park';
                $data['exten']   = $thisexten;
                $extenlist[$data['channel']]  = $data;
            }
        }
    } else {
        $results= $db->select("if(`defaultlot`='no',concat('PARK/parkinglot_',id),'PARK/default') AS channel,parkext AS exten,id,name","$database.parkplus");
        if(is_array($results)){
            foreach($results as $result) {
                $thisexten = $result['exten'];
                $data = array();
                $data['name']   = ucfirst($result['name']);
                $data['channel'] = $result['channel'];
                $data['type']    = 'park';
                $data['exten']   = $thisexten;
                $extenlist[$result['channel']]  = $data;
            }
        }
    }
    return $extenlist;

}

function queues_check_extensions_freepbx($exten=true) {
    global $active_modules, $db;

    $extenlist = array();
    if (is_array($exten) && empty($exten)) {
        return $extenlist;
    }
 
    $fpbx_version = preg_replace('/[^0-9]/','',freepbx_get_version());
    if($fpbx_version <= 260) {
        $context="from-internal";
    } else {
        $context="from-queue";
    }

    $results = $db->consulta("DESC queues_config");
    if(isset($results)) {
        # FreePBX 2.8 o superior
        $queues_table = "queues_config";
        $where='';
        $orderby = "extension";
        if (is_array($exten)) {
           $where = "extension in ('".implode("','",$exten)."')";
        }
    } else {
        # FreePBX Viejo 
        $queues_table = "extensions";
        $orderby = "extension";
        $where = "application='Queue' ";
        if (is_array($exten)) {
            $where .= "AND extension in ('".implode("','",$exten)."')";
        }
    }
    
    $results = $db->select("extension,descr",$queues_table,"",$where,"","",$orderby);

    if(is_array($results)) {
        foreach ($results as $result) {
            $thisexten = $result['extension'];
            $data = array();
            $data['name'] = $result['descr'];
            $data['channel'] = "QUEUE/$thisexten";
            $data['context'] = $context;
            $data['type'] = 'queue';
            $data['exten']   = $thisexten;
            $extenlist[$data['channel']]  = $data;
        }
    }

    return $extenlist;
}

function read_voicemail_conf_pins($file) {
    $voicemail_pin = array();
    if(is_readable($file)) {
        $vmf = @file($file);
        $vmctx = '';
        foreach($vmf as $linea) {
            $linea=trim($linea);
            if(preg_match("/^\[/",$linea)) {
                $vmctx = substr($linea,1,-1);
            }
            if(preg_match("/^\d/",$linea)) {
                $partes = preg_split("/=>/",$linea);
                $vmext = trim($partes[0]);
                $pertes = preg_split("/,/",$partes[1]);
                $vmpas = trim($pertes[0]);
                $voicemail_pin["$vmext@$vmctx"]=$vmpas;
            }
        }
    }
    return $voicemail_pin;
} 

function fop2_add_group($post) {

    global $db;
    extract($post);

    $name = trim(mysql_real_escape_string($name));

    if ( fop2_chk_group($name) ) {
        echo "<script>javascript:alert(\"" . __("The group name is already in use.") . "\"); history.back(-1);</script>";
        return false;
    }

    if ( $name =='' ) {
        echo "<script>javascript:alert(\"" . __("The name cannot be empty.") . "\"); history.back(-1);</script>";
        return false;
    }

    $results = $db->consulta("INSERT INTO fop2groups (name) VALUES ('$name')");

    if($results==1) {
        $results = $db->select("id,device","fop2buttons");
        if(is_array($results)){
            foreach($results as $result){
                $butid[$result['device']]=$result['id'];
            }
        }

        if(isset($post['includebot'])) {
            foreach($post['includebot'] as $idx=>$chan){
                $results = $db->consulta("INSERT INTO fop2GroupButton (group_name,id_button) VALUES ('$name', '".$butid[$chan]."')");
            }
        }
    }

    return true;
}

function fop2_get_group($id) {
    global $db;
    $results = $db->select("*","fop2groups","","id='$id'");
    return $results[0];
}

function fop2_del_button($id,$exten) {
    global $db;
    $id = preg_replace('/[^\d]/','',$id);
    if($exten<>"") {
        $results = $db->consulta("DELETE FROM fop2UserGroup WHERE exten = '$exten'");
        $results = $db->consulta("DELETE FROM fop2UserPlugin WHERE exten = '$exten'");
        $results = $db->consulta("DELETE FROM fop2users WHERE exten = '$exten'");
    }
    $results = $db->consulta("DELETE FROM fop2GroupButton WHERE id_button = '$id'");
    $results = $db->consulta("DELETE FROM fop2buttons WHERE id = '$id'");
    return true;
}

function fop2_del_group($id) {
    global $db;
    $id = preg_replace('/[^\d]/','',$id);
    $thisItem = fop2_get_group($id);
    $name = $thisItem['name'];
    $results = $db->consulta("DELETE FROM fop2GroupButton WHERE group_name = '$name'");
    $results = $db->consulta("DELETE FROM fop2groups WHERE id = '$id'");
    $results = $db->consulta("DELETE FROM fop2UserGroup WHERE id_group = '$id'");

    // Borramos grupo de template
    $results = $db->select("*","fop2templates","","groups LIKE '%$name%'");
    if(is_array($results)){
        foreach($results as $result){
            $grupejos = preg_split("/,/",$result['groups']);
            $finalgrp = array();
            foreach($grupejos as $itemgroup) {
                if($itemgroup<>$name) {
                    $finalgrp[]=$itemgroup;
                }
            }
            $finalgroupstring = implode(",",$finalgrp);
            $db->consulta("UPDATE fop2templates SET groups='$finalgroupstring' WHERE id='".$result['id']."'");
        }
    }
    return true;
}

function fop2_edit_group($id,$post) {
    global $db;
    $id = preg_replace('/[^\d]/','',$id);
    $thisItem = fop2_get_group($id);
    $name = $thisItem['name'];
    extract($post);

    $results = $db->consulta("DELETE FROM fop2GroupButton WHERE group_name = '$name'");
    $results = $db->select("id,device","fop2buttons");
    if(is_array($results)){
        foreach($results as $result){
            $butid[$result['device']]=$result['id'];
        }
    }

    foreach($post['includebot'] as $idx=>$chan){
        $results = $db->consulta("INSERT INTO fop2GroupButton (group_name,id_button) VALUES ('$name', '".$butid[$chan]."')");
    }
    return true;
}

function fop2_list_groups($all) {
    global $db;
    if($all==1) {
        $results = $db->select("*","fop2groups","","id>=-9","name");
    } else {
        $results = $db->select("*","fop2groups","","id>=0","name");
    }
    if(is_array($results)){
        foreach($results as $result){
            $allowed[] = $result;
        }
    }
    if (isset($allowed)) {
        return $allowed;
    } else {
        return array();
    }
}

function fop2_list_plugins() {
    global $db;
    $results = $db->select("*","fop2plugins","","","name");
    if(is_array($results)){
        foreach($results as $result){
            $allowed[] = $result;
        }
    }
    if (isset($allowed)) {
        return $allowed;
    } else {
        return array();
    }
}

function fop2_get_group_buttons($name) {
    global $db;
    $ret = array();
    $results = $db->select("id_button","fop2GroupButton","","group_name='$name'");
    if(is_array($results)){
        foreach($results as $idx => $cols) {
            $ret[]=$cols['id_button'];
        }
        return $ret;
    } else {
        return array();
    }
}

function fop2_set_button_order($vars) {
    global $db;
    foreach($vars['listitem'] as $orden=>$typeexten) {
        $partes = preg_split("/!/",$typeexten);
        $exten  = $partes[1];
        $type   = $partes[0];
        $exten = preg_replace("/^trunkout/","OUT_",$exten);
        $query="UPDATE fop2buttons SET sortorder='$orden' WHERE exten='$exten' AND type='$type'";
        $db->consulta($query);
    }
}

function getFastestMirror() {
   $mirror = array();
   $mirror[]="download.fop2.com";
   $mirror[]="download2.fop2.com";
   $mirror[]="download3.fop2.com";
   $faster = 1000;
   $use = $mirror[0];
   foreach($mirror as $host) {
      $result = ping($host,1);
      if($result<$faster) { $use = $host; $faster=$result; }
   }
   return $use;
}

function plugin_get_online($mirror) {
    $return = array();
    $fn = "http://$mirror/plugins/plugins.xml";
    ini_set('user_agent','Wget/1.10.2 (Red Hat modified)');
    $available_online = @ file_get_contents($fn);
    $xml = new xml2Array();
    $xmlarray = $xml->parseAdvanced($available_online);
    return $xmlarray['plugins']['plugin'];
}

function plugin_get_global() {
    global $db;
    $ret = array();
    $results = $db->select("rawname","fop2plugins","","global=1");
    if(is_array($results)){
        foreach($results as $idx => $cols) {
            $ret[$cols['rawname']]=1;
        }
    } 
    return $ret;
}

function get_installed_plugins($field='') {
    $results = array();
    $directory = dirname(__FILE__)."/plugins/";
    if(!is_dir($directory)) {
       mkdir($directory);
    }
    if ($handle = opendir($directory)) {
        while (false !== ($entry = readdir($handle))) {
            if ($entry != "." && $entry != "..") {
                if(is_dir($directory.$entry)) {
                     $data = file_get_contents($directory.$entry.'/plugin.xml');
                     $xml = new xml2Array();
                     $xmlarray = $xml->parseAdvanced($data);
                     if($field<>'') {
                         $results[$xmlarray['plugin']['rawname']]=$xmlarray['plugin'][$field];
                     } else {
                         $results[$xmlarray['plugin']['rawname']]=$xmlarray['plugin'];
                     }
                }
            }
        }
        closedir($handle);
    }
    return $results;
}

function plugin_read_xml_params($rawname) {
    $directory = dirname(__FILE__)."/plugins/";
    if(is_readable($directory.$rawname.'/plugin.xml')) {
        $data = file_get_contents($directory.$rawname.'/plugin.xml');
        $xml = new xml2Array();
        $xmlarray = $xml->parseAdvanced($data);
    }
    $ret = isset($xmlarray['plugin']['params'])?$xmlarray['plugin']['params']:array();
    return $ret;
}

function plugin_insert_missing_db() {
    // Inserta datos de plugins localmente instalados en la tabla fop2plugins 
    global $db;
    $plugsondb = array();
    $plugin_db = fop2_list_plugins();
    foreach($plugin_db as $idx=>$data) {
        $plugsondb[] = $data['rawname'];
    }
    $plugin_local = get_installed_plugins();
    foreach($plugin_local as $rawname=>$data) {
        if(!in_array($rawname,$plugsondb)) {
            if($data['global']=="yes") { $global=1; }
            $results = $db->consulta( "INSERT INTO fop2plugins (rawname, name, version, description, global) values ('%s','%s','%s','%s', '%s') ", 
                                  array( $data['rawname'] ,$data['name'], $data['version'], $data['description'], $global ) );
        }
    }
}

function fop2_rrmdir($dir) {

    if(!preg_match('/\/plugins/',$dir) && !preg_match('/\/_cache/',$dir)) { die("no dice $dir"); }

    foreach(glob($dir . '/*') as $file) {
        if(is_dir($file)) { fop2_rrmdir($file); } else { unlink($file); }; 
    }
    rmdir($dir); 
}

function plugin_delete($itemid) {
    global $db;
    $itemid = preg_replace("/[^A-Za-z0-9\._-]/","",$itemid);
    $rawname = $itemid;
    $directory = dirname(__FILE__)."/plugins/";
    $deldir = substr(escapeshellarg($directory.$rawname),1,-1);
    fop2_rrmdir($deldir);
    $results = $db->consulta( "DELETE FROM fop2plugins WHERE rawname='".$db->escape_string($rawname)."'");
    $results = $db->consulta( "DELETE FROM fop2UserPlugin WHERE id_plugin='".$db->escape_string($rawname)."'");

}

function plugin_handleupload($uploaded_file) {

    $directory = dirname(__FILE__)."/plugins/";
    $errors = array();
        
    if (!isset($uploaded_file['tmp_name']) || !file_exists($uploaded_file['tmp_name'])) {
        $errors[] = __("Error finding uploaded file. Did you select one? Check your PHP and/or web server configuration.");
        return $errors;
    }
        
    if (!preg_match('/\.(tar\.gz|tgz)$/', $uploaded_file['name'])) {
        $errors[] = __("File must be in tar+gzip (.tgz or .tar.gz) format");
        return $errors;
    }
        
    if (!preg_match('/^([A-Za-z][A-Za-z0-9_]+)\-([0-9a-zA-Z]+(\.[0-9a-zA-Z]+)*)\.(tar\.gz|tgz)$/', $uploaded_file['name'], $matches)) {
        $errors[] = __("Filename does not have the correct format: it must be pluginname-version.tar.gz (eg. customplugin-0.1.tar.gz)");
        return $errors;
    } else {
        $pluginname    = $matches[1];
        $pluginversion = $matches[2];
    }
        
    $temppath = $directory.'/_cache/'.uniqid("upload");
    if (! @mkdir($temppath) ) {
        return array(sprintf(__("Error creating temporary directory: %s"), $temppath));
    }
    $filename = $temppath.'/'.$uploaded_file['name'];
        
    move_uploaded_file($uploaded_file['tmp_name'], $filename);
        
    exec("tar ztf ".escapeshellarg($filename), $output, $exitcode);
    if ($exitcode != 0) {
        $errors[] = __("Error untaring uploaded file. Must be a tar+gzip file");
        return $errors;
    }
    foreach ($output as $line) {
        // make sure all lines start with "pluginname/"
        if (!preg_match('/^'.$pluginname.'\//', $line)) {
            $errors[] = 'File extracting to invalid location: '.$line;
        }
    }
    if (count($errors)) {
        return $errors;
    }

    exec("rm -rf ".$directory."/_cache/$pluginname", $output, $exitcode);
    if ($exitcode != 0) {
        return array(sprintf(__('Could not remove %s to install new version'), $directory.'/_cache/'.$pluginnam));
    }
    exec("tar zxf ".escapeshellarg($filename)." -C ".escapeshellarg($directory.'/_cache/'), $output, $exitcode);
    if ($exitcode != 0) {
        return array(sprintf(__('Could not untar %s to %s'), $filename, $directory.'/_cache'));
    }
    exec("rm -rf ".$directory.$pluginname, $output, $exitcode);
    if ($exitcode != 0) {
        return array(sprintf(__('Could not remove old plugin %s to install new version'), $directory.$pluginname));
    }
    exec("mv ".$directory."/_cache/$pluginname ".$directory.$pluginname, $output, $exitcode);
    if ($exitcode != 0) {
        return array(sprintf(__('Could not move %s to %s'), $directory."/_cache/$pluginname", $directory));
    }

    return true;
}

function plugin_get_licensed() {
    $timeout   = 5;
    $host      = "localhost";
    $context   = "GENERAL";
    $port      = "4445";
    $dati      = "";

    $sk=fsockopen($host,$port,$errnum,$errstr,$timeout) ;
    if (!is_resource($sk)) {
        return '0';
    } else {
        fputs($sk, "<msg data=\"$context|plugins|0|0\" />\0");
        stream_set_timeout($sk, 2);
        $dati.=fgets ($sk, 2048);
    }
    fclose($sk);
    $dati=preg_replace("/{.*}/","",$dati);
    $dati=preg_replace("/[^a-zA-Z,]/","",$dati);
    return $dati;
}

function fop2_print_config_form($rawname) {

    $directory = dirname(__FILE__)."/plugins/";
    $inifile = $directory."$rawname/$rawname.ini";

    if(file_exists($inifile)) {
        $ini_array = parse_ini_file($inifile, true);
    } else {
        $ini_array = array();
    }

    $convertToArray = 0;
    $multiform      = array();
    $multiform_type = array();
    $allpars        = plugin_read_xml_params($rawname);

    $ret="<h3>".__('Settings')."</h3><form id='plugin_$rawname' class='form-horizontal'>\n";
    $ret.="<fieldset style='display:none;' id='form-$rawname-skeleton'>\n";
    $ret.="<legend><span id='legend-".$rawname."-skeleton'></span></legend>\n";

    foreach ( $allpars['param'] as $key=>$param ) {
        if(!is_array($param)) {
           $convertToArray=1;
           break;
        }
    }

    if($convertToArray==1) {
        $tempA              = $allpars['param'];
        $allpars['param']   = array();
        $allpars['param'][] = $tempA;
    }


    foreach ( $allpars['param'] as $key=>$param ) {

       $formname          = $param['name'];
       $formtype          = $param['type'];
       $formdefault       = $param['default'];
       $formmulti         = $param['multi'];
       $fftype            = array();
       $fftype['string']  = 'text';
       $fftype['integer'] = 'number';

       $ret.="<div class='form-group'>\n";
       $ret.="<label class='col-sm-2 control-label' for='$rawname-".$formname."-skeleton'>$formname:</label>";
       $ret.="<div class='col-sm-10'>\n";
       $ret.="<input type='".$fftype[$formtype]."' name='$rawname-".$formname."-skeleton' id='$rawname-".$formname."-skeleton' value='".$formdefault."' class='form-control'>";
       $ret.="</div>\n";
       $ret.="</div>\n";
    }

    $ret.="<button id='removesection-$rawname-skeleton' onClick='fop2_removeSection(this.id); return false;' class='btn btn-warning'>".__('Remove Section')."</button><br/>\n";
    $ret.="</fieldset>\n\n";

    $ret.="<fieldset id='form-$rawname-default'>\n";
    foreach ( $allpars['param'] as $key=>$param ) {
       $formname          = $param['name'];
       $formtype          = $param['type'];
       $formdefault       = $param['default'];
       $formmulti         = $param['multi'];
       $fftype            = array();
       $fftype['string']  = 'text';
       $fftype['integer'] = 'number';
       $formval           = (isset($ini_array[$formname]))?$ini_array[$formname]:$formdefault;

       $ret.="<div class='form-group'>\n";
       $ret.="<label class='col-sm-2 control-label' for='$rawname-".$formname."-default'>$formname:</label>";
       $ret.="<div class='col-sm-10'>\n";
       $ret.="<input class='form-control' type='".$fftype[$formtype]."' ";
       $ret.="name='$rawname-".$formname."-default' id='$rawname-".$formname."-default' value='".$formval."'>";
       $ret.="</div>\n";
       $ret.="</div>\n";

       if($formmulti=='yes') {
           $multiform[$formname]      = $formdefault;
           $multiform_type[$formname] = $formtype;
       }
    }
    $ret.="</fieldset>\n";

    foreach($ini_array as $section=>$val) {
        if(is_array($val)) {
            $ret.="<fieldset id='form-$rawname-$section'><legend style='border-top:1px solid #aaa;'>$section</legend>\n";
            foreach($multiform as $formname=>$formdefault) {
                $formtype = $multiform_type[$formname];
                $formval = (isset($ini_array[$section][$formname]))?$ini_array[$section][$formname]:$formdefault;

                $ret.="<div class='form-group'>\n";
                $ret.="<label class='col-sm-2 control-label' for='$rawname-".$formname."-$section'>$formname:</label>";
                $ret.="<div class='col-sm-10'>\n";
                $ret.="<input class='form-control' type='".$fftype[$formtype]."' name='$rawname-".$formname."-$section' id='$rawname-".$formname."-$section' value='".$formval."'>";
                $ret.="</div>\n"; 
                $ret.="</div>\n";
            }
            $ret.="<button class='btn btn-warning' id='removesection-$rawname-$section' onClick='fop2_removeSection(this.id); return false;'>".__('Remove Section')."</button><br/>";
            $ret.="</fieldset>\n";
        }
    }
    $ret.="</form>\n<hr style='border-top-color:#aaa; margin-top:10px; margin-bottom:10px;'/>";

    if(count($multiform)>0) {
        $ret.="<button class='btn btn-primary' onClick='fop2_addConfig(\"$rawname\",\"plugin_$rawname\",\"".__('Enter Section')."\",\"".__("That section already exists!")."\",\"".__("Section cannot be empty!")."\"); return false;'>".__('Add Section')."</button>";
    }
    $ret.=" <button class='btn btn-primary' onClick='fop2_saveConfig(\"$rawname\",\"".__("Configuration saved")."\"); return false;'>".__('Save')."</button>";
    return $ret;
}

function ping($host,$timeout=1) {
    /* ICMP ping packet with a pre-calculated checksum */
    $ip = gethostbyname($host);

    $output = array();
    $com = 'ping -n -w ' . $timeout . ' -c 1 ' . $ip;
    
    $exitcode = 0;

    exec($com, $output, $exitcode);

    if ($exitcode == 0 || $exitcode == 1)
    { 
        foreach($output as $cline)
        {
            if (strpos($cline, ' bytes from ') !== FALSE)
            {
                $out = (int)ceil(floatval(substr($cline, strpos($cline, 'time=') + 5)));
                return $out;
            }
        }
    }
    
    return 1000;
}

function plugin_download($itemid, $mirror) {
    global $db;

    $directory = dirname(__FILE__)."/plugins/";
    $itemid = preg_replace("/[^A-Za-z0-9\._-]/","",$itemid);
    $fn = "http://$mirror/plugins/$itemid.tgz";

    $pluginjs = @ file_get_contents($fn);
    $tgzfile = substr(escapeshellarg($directory."$itemid.tgz"),1,-1);

    $fp = fopen($tgzfile,"w");
    fputs($fp,$pluginjs);
    fclose($fp);

    exec("tar zxf ".$tgzfile." -C ".escapeshellarg($directory), $output, $exitcode);
    unlink($tgzfile);

    if($exitcode==0) {
        // Insertar en la base de datos el plugin
        list ( $rawname, $nada) = preg_split("/-/",$itemid,2);
        $infoxml = substr(escapeshellarg($directory."$rawname/plugin.xml"),1,-1);
        $pluginxml = file_get_contents($infoxml);
        $xml = new xml2Array();
        $xmlarray = $xml->parseAdvanced($pluginxml);
        $data = $xmlarray['plugin'];
        if($data['global']=='yes') { $global=1; } else { $global=0; }
        $results = $db->consulta( "REPLACE INTO fop2plugins (rawname, name, version, description, global) values ('".$db->escape_string($data['rawname'])."','".$db->escape_string($data['name'])."','".$db->escape_string($data['version'])."','".$db->escape_string($data['description'])."','$global') ");
        die('OK');
    } else {
        die("ERROR1 $exitcode");
    }
}

function fop2manager_core_download($itemid, $mirror) {

    global $db;

    $directory = dirname(__FILE__)."/_cache/";

    if(!is_dir($directory)) {
        mkdir($directory);
    }

    $itemid = preg_replace("/[^A-Za-z0-9\._-]/","",$itemid);
    $fn = "http://$mirror/plugins/$itemid.tgz";

    $pluginjs = @ file_get_contents($fn);
    $tgzfile = substr(escapeshellarg($directory."$itemid.tgz"),1,-1);

    echo "tgz $tgzfile, dir $directory<br>";

    $fp = fopen($tgzfile,"w");
    fputs($fp,$pluginjs);
    fclose($fp);

    exec("tar zxf ".$tgzfile." -C ".escapeshellarg($directory), $output, $exitcode);
    unlink($tgzfile);

    if($exitcode==0) {
        // Actualizar
        die('OK');
    } else {
        die("ERROR1 $exitcode");
    }
}

function freepbx_get_version() {
    global $conf,$db;
    $database = $conf['DBNAME'];
    $result = $db->select("value","$database.admin","","variable='version'");
    if(is_array($result)) {
        return $result[0];
    }
}

function freepbx_get_queues() {

    global $db,$astman,$conf,$fpbx_version;
    $return = array();
    $database = $conf['DBNAME'];

    $fpbx_version = preg_replace('/[^0-9]/','',freepbx_get_version());
    if($fpbx_version <= 260) {
        $context="from-internal";
    } else {
        $context="from-queue";
    }

    if(!$res = $astman->connect("127.0.0.1:".$conf["MGRPORT"], $conf["MGRUSER"] , $conf["MGRPASS"], 'off')) {
        unset($astman);
    }

    $results = $db->select("id,keyword,data","$database.queues_details","","keyword in ('strategy','member')");
    foreach($results as $result) {
        if(!isset($return[$result['id']])) {
            $return[$result['id']]=array();
        }
        if($result['keyword']=='member') { 
            $return[$result['id']]['members'][]=$result['data']; 
        }
    }

    if ($astman) {
        foreach($return as $queue=>$data) {
            $dynmem = Array();
            $mem    = Array();
            $get=$astman->database_show('QPENALTY/'.$queue.'/agents');
            if ($get) {
                foreach($get as $key => $value){
                    $key=explode('/',$key);
                    $mem[$key[4]]=$value;
                }
                foreach($mem as $mem => $pty){
                    $dynmem[]=freepbx_construct_member($mem,$pty,$context);
                }
                $return[$queue]['dynmembers']=$dynmem;

            } else {
               $return[$queue]['dynmembers']=array();
            }

            $return[$queue]['dynmemberonly'] = $astman->database_get('QPENALTY/'.$queue,'dynmemberonly');
        }
    }



    return $return; 
}

function freepbx_construct_member($mem,$pty,$context) {

    $return       = '';
    $exten_prefix = strtoupper(substr($mem,0,1));
    $this_member  = preg_replace("/[^0-9#\,*]/", "", $mem);

    switch ($exten_prefix) {
        case 'A':
          $exten_type = 'Agent';
          break;
        case 'S':
          $exten_type = 'SIP';
          break;
        case 'X':
          $exten_type = 'IAX2';
          break;
        case 'Z':
          $exten_type = 'ZAP';
          break;
        case 'D':
          $exten_type = 'DAHDI';
          break;
        default;
          $exten_type = 'Local';
    }

    // remove blanks // prefix with the channel
    if (!empty($this_member)) {
        switch($exten_type) {
            case 'Agent':
            case 'SIP':
            case 'IAX2':
            case 'ZAP':
            case 'DAHDI':
                $return = "$exten_type/$this_member,$pty";
                break;
            case 'Local':
                $return = "$exten_type/$this_member@$context/n,$pty";
                break;
        }
    }
    return $return;
}

function freepbx_construct_queuechannel($extension,$device,$name,$qcontext,$queue_conf) {

    // queuechannel=
    //    Local/602@from-queue/n|Penalty=0|MemberName=Nicolas Gudino 1|StateInterface=SIP/602
    //   &Local/602@from-queue/n|Penalty=0|MemberName=Nicolas Gudino 1|StateInterface=SIP/602|Queue=100
    //   &Local/602@from-queue/n|Penalty=0|MemberName=Nicolas Gudino 1|StateInterface=SIP/602|Queue=101

    $member = array();

    foreach($queue_conf as $queue=>$data) {

        $allmembers = array();
        if(isset($data['dynmembers'])) {
            $allmembers = array_merge($data['dynmembers'],$allmembers);
        }
        if(isset($data['members'])) {
            $allmembers = array_merge($data['members'],$allmembers);
        }

        foreach($allmembers as $idx=>$mem) {
            $pen_pos = strrpos($mem, ",");
            if ( $pen_pos !== false ) {
                $thisexten = substr($mem,0,$pen_pos); 
            } else {
                $thisexten = $mem; 
            }
            $thisexten = preg_replace("/[^0-9#*]/", "", $thisexten); 

            if($extension==$thisexten) {
                $member[$queue][$thisexten]=$mem."|".$device;
            }
        }

    }
    if(substr($device,0,6)=="VIRTUA" || substr($device,0,6)=="CUSTOM") {
        $stint = "";
    } else {
        $stint = "|StateInterface=$device";
    }
    $def[] = "Local/$extension@$qcontext/n|Penalty=0|MemberName=$name".$stint;
    foreach($member as $queue=>$datos) {
        foreach($datos as $exten=>$mem) {
            $partos = preg_split("/\|/",$mem);
            $partes = preg_split("/,/",$partos[0]);
            $dev    = $partes[0];
            $pen    = (isset($partes[1]))?$partes[1]:0;
            $def[] = "$dev|Penalty=$pen|MemberName=$name".$stint."|Queue=$queue";
        }
    }
    $return = implode("&",$def);
    return $return;
}

function parse_conf($filename) {
    global $config_engine;

    $file = file($filename);

    foreach ($file as $line) {
        if (preg_match("/^\s*([\w]+)\s*=\s*\"?([\w\/\:\.\*\%-]*)\"?\s*([;#].*)?/",$line,$matches)) {
            $matches[1] = preg_replace("/^AMP/","",$matches[1]);
            if($config_engine=='thirdlane_db') {
                if($matches[1]=='DBCONFIG') { $matches[1]='DBNAME'; $conf['DBHOST']='localhost';} // Convert thirdlane db name entry
            }
            $conf[ $matches[1] ] = $matches[2];
        }
    }

    if($config_engine=='thirdlane_db' || $config_engine=='thirdlane_old') {
        $conf['MGRPORT']='5038';
        $conf['MGRUSER']='internal';
        $conf['MGRPASS']='insecure';
    }

    if(!isset($conf['MGRPORT'])) {
        $conf['MGRPORT']='5038';
    }

    if(!isset($conf['MGRHOST'])) {
        $conf['MGRHOST']='127.0.0.1';
    }
    return $conf;
}

function fop2_permissions() {
    // Returns list of fop2 permissions as an array
    $perm = array( 'all', 'dial', 'hangup', 'hangupself', 'meetme', 
                   'pickup', 'record', 'recordself', 'spy', 'transfer', 
                   'transferexternal', 'queuemanager', 'queueagent', 
                   'phonebook','chat','broadcast', 'preferences',
                   'voicemailadmin','sms','smsmanager'
            );
    asort($perm);
    return $perm;
}

function fop2manager_error($errno, $errstr, $errfile, $errline) {
    global $DEBUG;

    switch ($errno) {
        case E_USER_ERROR:
            $extraclass='danger';
            break;
        case E_USER_WARNING:
            $extraclass='warning';
            break;
        case E_USER_NOTICE:
            $extraclass='info';
            break;
        default:
           $extraclass="info";
           break;
    }
    if($DEBUG==0 && $extraclass=='info') {  return; }
    echo "<div class='alert alert-$extraclass'>";
    echo "<pre>$errfile (line $errline)</h2>\n$errstr</pre>";
    echo "</div>\n";
}

function reload_asterisk() {
    global $astman, $conf;

    if(!$res = $astman->connect($conf['MGRHOST'].':'.$conf['MGRPORT'], $conf['MGRUSER'] , $conf['MGRPASS'], 'off')) {
        unset($astman);
    }

    if ($astman) {
        $res = $astman->reload();
    }
}

// Used for natural sorting or arrays
function naturalsort($a, $b) {
    return strnatcmp($a, $b);
}


/**
* Create a new directory, and the whole path.
*
* If  the  parent  directory  does  not exists, we will create it,
* etc.
* @todo
*     - PHP5 mkdir functoin supports recursive, it should be used
* @author baldurien at club-internet dot fr 
* @param string the directory to create
* @param int the mode to apply on the directory
* @return bool return true on success, false else
* @previousNames mkdirs
*/

function makeAll($dir, $mode = 0777, $recursive = true) {
    if( is_null($dir) || $dir === "" ){
        return FALSE;
    }
    
    if( is_dir($dir) || $dir === "/" ){
        return TRUE;
    }
    if( makeAll(dirname($dir), $mode, $recursive) ){
        return mkdir($dir, $mode);
    }
    return FALSE;
}

/**
 * Copies file or folder from source to destination, it can also do
 * recursive copy by recursively creating the dest file or directory path if it wasn't exist
 * Use cases:
 * - Src:/home/test/file.txt ,Dst:/home/test/b ,Result:/home/test/b -> If source was file copy file.txt name with b as name to destination
 * - Src:/home/test/file.txt ,Dst:/home/test/b/ ,Result:/home/test/b/file.txt -> If source was file Creates b directory if does not exsits and copy file.txt into it
 * - Src:/home/test ,Dst:/home/ ,Result:/home/test/** -> If source was directory copy test directory and all of its content into dest      
 * - Src:/home/test/ ,Dst:/home/ ,Result:/home/**-> if source was direcotry copy its content to dest
 * - Src:/home/test ,Dst:/home/test2 ,Result:/home/test2/** -> if source was directoy copy it and its content to dest with test2 as name
 * - Src:/home/test/ ,Dst:/home/test2 ,Result:->/home/test2/** if source was directoy copy it and its content to dest with test2 as name
 * @author Sina Salek (<a href="http://sina.salek.ws/node/1289" title="http://sina.salek.ws/node/1289">http://sina.salek.ws/node/1289</a>)
 * @todo
 *  - Should have rollback so it can undo the copy when it wasn't completely successful
 *  - It should be possible to turn off auto path creation feature f
 *  - Supporting callback function
 *  - May prevent some issues on shared enviroments : <a href="http://us3.php.net/umask" title="http://us3.php.net/umask">http://us3.php.net/umask</a>
 * @param $source //file or folder
 * @param $dest ///file or folder
 * @param $options //folderPermission,filePermission
 * @return boolean
 */
function smartCopy($source, $dest, $options=array('folderPermission'=>0755,'filePermission'=>0755))
{
    $result=false;
    
    //For Cross Platform Compatibility
    if (!isset($options['noTheFirstRun'])) {
        $source=str_replace('\\','/',$source);
        $dest=str_replace('\\','/',$dest);
        $options['noTheFirstRun']=true;
    }
    
    if (is_file($source)) {
        if ($dest[strlen($dest)-1]=='/') {
            if (!file_exists($dest)) {
                makeAll($dest,$options['folderPermission'],true);
            }
            $__dest=$dest."/".basename($source);
        } else {
            $__dest=$dest;
        }
        $path_parts = pathinfo($__dest);
        if($path_parts['extension']=='ini') {
            //echo "skip ini file $__dest<br>";
        } else {
            $result=copy($source, $__dest);
            chmod($__dest,$options['filePermission']);
        }
    } elseif(is_dir($source)) {
        if ($dest[strlen($dest)-1]=='/') {
            if ($source[strlen($source)-1]=='/') {
                //Copy only contents
            } else {
                //Change parent itself and its contents
                $dest=$dest.basename($source);
                if(!is_dir($dest)) { @mkdir($dest); }
                chmod($dest,$options['filePermission']);
            }
        } else {
            if ($source[strlen($source)-1]=='/') {
                //Copy parent directory with new name and all its content
                if(!is_dir($dest)) { @mkdir($dest,$options['folderPermission']); }
                chmod($dest,$options['filePermission']);
            } else {
                //Copy parent directory with new name and all its content
                if(!is_dir($dest)) { @mkdir($dest,$options['folderPermission']); }
                chmod($dest,$options['filePermission']);
            }
        }

        $dirHandle=opendir($source);
        while($file=readdir($dirHandle))
        {
            if($file!="." && $file!="..")
            {
                $__dest=$dest."/".$file;
                $__source=$source."/".$file;

                $pos = strpos($__source,"_cache");
                $resto = substr($__source,$pos+6);
                if(preg_match("/_cache/",$resto)) {
                   // echo "skip $__source<br>";
                } else {
                    if ($__source!=$dest) {
                        $result=smartCopy($__source, $__dest, $options);
                    }
                }
            }
        }
        closedir($dirHandle);
        
    } else {
        $result=false;
    }
    return $result;
}
